unit Aritmet;
{
konstanty
  - pocet kroku vykreslovani funkce
  - desetinna tecka, unarni minus

funkce pro
  - zpracovani vstupniho vyrazu (rozrezani na tokeny, prevod infix na strom)
  - vraceni hodnoty funkce v bode x
}

interface

uses
  Strings;

const
  KROKU = 300;  { pocet kroku pri vykreslovani, cim vyssi cislo tim hladsi graf }
  DTECKA = '.';       { desetinna tecka }
  UNARNIMINUS = '~';  { unarni minus }
  MAXDVYRAZ = 3; { maximalni delka subvyrazu v matematickem vyrazu: 'cos'=3 }
  MAXVYRAZU = 50;     { maximalni pocet subvyrazu v jednom vyrazu }
  MAXLENZAP = 80;     { maximalni delka zapisu }
  PREDPISKONEC = '$'; { ukonceni predpisu funkce - pro vnitrni potreby }
  FHODNOTA = '&';     { priznak pro vyskyt hodnoty v zapise funkce }
  EPS = 0.0001;       { odchylka pro porovnavani realnych cisel }
  MAXMOCNIN = 100;    { zabraneni behove chyby preteceni floating pt. operace }

type
  pzobrazeni = ^zobrazeni;
  zobrazeni = record       { jeden bod funkce }
    x: real;               { bod funkce }
    y: real;               { hodnota v bode x }
    def: Boolean;          { je hodnota definovana? }
    dalsi: pzobrazeni;     { pointer na dalsi bod }
  end;
  zClen = record    { zapis funkce tvori cisla nebo operatory,
                      pripadne funkce (sin, cos, ...) }
    bZnak: boolean; { true - koukej na znak .. dalo by se resit
                      pascalovskou konstrukci }
    hodnota: real;  { slo-li o cislo, zde je jeho hodnota }
    znak: char;     { pouzivame takove kodovani kde kazdy subvyraz
                      lze nahradit jednim znakem }
  end;
  zZapis = array[1..MAXVYRAZU] of zClen;  { zapis je pole subvyrazu }
  funkce = record                         { 'objekt' funkce }
    sPredpis: array[0..MAXLENZAP] of char; { jeji zapis }
    sPredpisTokens: zZapis;                { zapis rozdeleny na subvyrazy }
    pZobr: pzobrazeni;                     { lin. spoj. seznam bodu }
  end;

function Umocni(x: real; y: longint; var def: boolean): real;
  { umocni cislo x na celociselny exponent y }

procedure Tokenize(var fce: funkce);
  { rozdeli infix zapis funkce na subvyrazy }

procedure ZiskejPrubeh(var fce: funkce; defMin: real; defMax: real);
  { naplni lin. spojovy seznam body prubehu funkce fce
    v intervalu (defMin-defMax) }

function Hodnota(var fce: funkce; x: real; var def: Boolean): real;
  { spocita hodnotu funkce fce v bode x, pripadne
    v def vrati False, neni-li tam fce definovana }

implementation

function IsNumeric(c: char): boolean;
begin
  IsNumeric := ((c>='0') and (c<='9')) or (c = DTECKA);
end;

procedure Tokenize(var fce: funkce);
{ rozreze vstupni zapis funkce na podvyrazy }
var
  i: integer; { pozice ve zpracovavanem predpisu }
  j: integer; { poradi ukladaneho subvyrazu }
  l: integer; { delka vstupniho predpisu }
  fZnak: integer; { priznak prave zpracovavaneho subvyrazu 0=cislo,
                    1=jednoznakovy operator, 3=trojznakovy operator (funkce) }
  s: String[MAXDVYRAZ]; { pro vytahnuti a prevod cisla z vyrazu do real promenne }
  code: Integer; { navratova hodnota funkce val }
begin
  l := strlen(fce.sPredpis);
  i := 0; j := 1;

  while i < l do begin
    case fce.sPredpis[i] of
    { toto jsou jednopismenne subvyrazy }
    'e','^','/','*','+','-',')','(','~','x','P': fZnak := 1;
    { toto jsou zacatky trojpismennych nazvu funkci
      takze vyuzivame toho, ze nejsou navzajem konfliktni
      kazda funkce je tak zastoupena svym prvnim znakem }
    's','c','t','a','l','o': fZnak := 3;
    { mezera muze slouzit pro lepsi formatovani zapisu }
    ' ': fZnak := -1;
    { jinak se jedna o cislo }
    else fZnak := 0;
    end;

    { jedna-li se o neciselnou hodnotu }
    if fZnak > 0 then begin
      { zapiseme si jeji (prvni) znak }
      fce.sPredpisTokens[j].znak := fce.sPredpis[i];
      fce.sPredpisTokens[j].bZnak := True;
      Inc(i, fZnak);
    { jedna-li se o ciselnou hodnotu }
    end else if fZnak = -1 then Inc(i)
    else begin
      s := '';
      { celou ji precteme do pascalovskeho stringu }
      while IsNumeric(fce.sPredpis[i]) do begin
        s := s + fce.sPredpis[i];
        Inc(i);
      end;
      { a prevedeme a ulozime }
      val(s,fce.sPredpisTokens[j].hodnota,code);
      fce.sPredpisTokens[j].znak := '@';
      fce.sPredpisTokens[j].bZnak := False;
      if (code > 0) then writeln('Chyba ve vstupnim vyraze na pozici ',i+code);
    end;
    { posun na dalsi subvyraz }
    If fZnak >= 0 then Inc(j);
  end;
  fce.sPredpisTokens[j].znak := PREDPISKONEC;
end;

procedure MError(hlaska: string);
{ vypise chybu a ukonci program }
begin
  writeln('Doslo k chybe: ',hlaska);
  Halt;
end;

function Umocni(x: real; y: longint; var def: boolean): real;
{ umocni cislo x na celociselny exponent y }
var
  r: real;
  d: longint;
  x2: real;
begin
  d := 2;
  x2 := x;
  def := True;
  if y < 0 then begin
    r := Umocni(x, -y, def);
    if def and (abs(r) > EPS) then
      Umocni := 1/r
    else
      def := False;
  end
  else if y = 0 then
    { cokoliv na nultou je jedna }
    Umocni := 1
  else if y = 1 then
    { x na prvou je x }
    Umocni := x
  else { y > 1 } begin
    while d <= y do begin
      x2 := x2 * x2;
      d := d * 2;
    end;
    { x := x ^ (d/2), kde d je mocnina dvou }
    Umocni := x2 * Umocni(x, y-(d div 2), def);
  end;
end;


function Hodnota(var fce: funkce; x: real; var def: boolean): real;
{ vrati hodnotu funkce fce v bode x, tedy f(x) }
var
  Znam: array[0..MAXVYRAZU] of char; { zasobnik operatoru }
  ZnamTop: 0..MAXVYRAZU;             { jeho vrchol }
  Zas: array[1..MAXVYRAZU] of real;  { zasobnik cisel }
  ZasTop: 0..MAXVYRAZU;              { jeho vrchol }
  r: real;        { cislo na vstupu }
  z: char;        { operator na vstupu }
  bCont: Boolean; { pokracovat }
  p: integer;     { pozice v nacitanem zapise }

  function Precti(var r: real; var z: char): Boolean;
  { vrati dalsi subvyraz ze zapisu funkce
    pouziva globalni promennou p jako akt. pozici v zapise }
  begin
    if fce.sPredpisTokens[p].znak = PREDPISKONEC then
      { konec vstupu }
      Precti := False
    else
      if fce.sPredpisTokens[p].bZnak then
        z := fce.sPredpisTokens[p].znak
      else begin
        { slo o hodnotu, nastavime priznak }
        z := FHODNOTA;
        r := fce.sPredpisTokens[p].hodnota
      end;
    { inkrementace globalniho ukazatele na pozici v predpise }
    Inc(p);
  end;

  procedure Operace(o: char);
  { provede operaci danou znakem o }
  var
    v, v1, v2: real;
  begin
    v := 0;
    v2 := Zas[ZasTop];
    { jde-li o binarni operator (se dvema operatory) }
    if (o in ['+','-','*','/','^']) then begin
      { nacteme dalsi operator (je v zasobniku niz => je prvni) }
      Dec(ZasTop); v1 := Zas[ZasTop];
    end;
    case o of
      '+': v := v1 + v2;
      '-': v := v1 - v2;
      '*': v := v1 * v2;
      '/':
        if abs(v2) < EPS then
        { v nule neni definovano }
          def := False
        else
          v := v1 / v2;
      '^': begin
        if abs(v2) > MAXMOCNIN then begin
        { ochrana pred runtime chybe: floating point operation overflow }
          def := False;
          Exit;
        end;
        if abs(v2-round(v2)) < EPS then
        { exponent je cele cislo }
          v := Umocni(v1,round(v2),def)
        else
        { exponent je desetinne cislo }
          if v1 < 0 then
            { pro zaporne zaklady neni definovano }
            def := false else
          if abs(v1) < EPS then
            { nula na cokoliv je vzdy nula }
            v := 0 else
          if abs(v1-1) < EPS then
            { jedna na cokoliv je vzdy jedna }
            v := 1 else
          if v1 > 0 then
            v := exp(v2*ln(v1))
      end;
      '~': v := -v2;
      'c': v := cos(v2);
      's': v := sin(v2);
      'a': v := abs(v2);
      't': v := sin(v2)/cos(v2);
      'l': if v2 > EPS then
        v := ln(v2)
      else
        { pro zaporne a nulove x neni logaritmus definovan }
        def := False;
      'o': begin
        if v2 < 0 then
          def := false
        else
          v := sqrt(v2);
      end;
    end; { of case }
    Zas[ZasTop] := v;
  end; { of procedure Operace }

begin
  ZasTop := 0;
  ZnamTop := 0;
  p := 1; def := True;
  bCont := Precti(r,z);

    while bCont and def do begin
      case z of
      FHODNOTA: begin
        { hodnotu ulozime na zasobnik operandu }
        Inc(ZasTop); Zas[ZasTop] := r
      end;
      'x': begin
        { promennou ulozime na zasobnik operandu }
        Inc(ZasTop); Zas[ZasTop] := x
      end;
      'e': begin
        { eulerovo cislo ulozime na zasobnik operandu }
        Inc(ZasTop); Zas[ZasTop] := exp(1.0);
      end;
      'P': begin
        { ludolfovo cislo ulozime na zasobnik operandu }
        Inc(ZasTop); Zas[ZasTop] := PI;
      end;
      { operace s nejvyssi prioritou }
      '(','s','c','a','l','t','o','~': begin
        Inc(ZnamTop); Znam[ZnamTop] := z;
      end;
      '^': begin
        while (ZnamTop > 0) and (Znam[ZnamTop] in ['s','c','a','l','t','o','~']) do begin
          Operace(Znam[ZnamTop]); Dec(ZnamTop); { ze zasobniku }
        end;
        Inc(ZnamTop); Znam[ZnamTop] := z;  { operaci na zasobnik }
      end;
      ')': begin
        { proved vsechny operace az po nejblizsi zavorku }
        while (ZnamTop > 0) and (Znam[ZnamTop] <> '(') do begin
          Operace(Znam[ZnamTop]); Dec(ZnamTop);
        end;
        if ZnamTop = 0 then
          MError('Chyba pri zpracovani zapisu funkce')
        else
          Dec(ZnamTop);
      end;
      '*', '/': begin
        { proved vsechny operace s vyssi a stejnou prioritou }
        while (ZnamTop > 0) and (Znam[ZnamTop] in ['*','/','s','c','a','l','t','o','^','~']) do begin
          Operace(Znam[ZnamTop]); Dec(ZnamTop); { ze zasobniku }
        end;
        Inc(ZnamTop); Znam[ZnamTop] := z;  { operaci na zasobnik }
      end;
      '+', '-': begin
        { proved vsechny operace s vyssi a stejnou prioritou }
        while (ZnamTop > 0) and (Znam[ZnamTop] in ['+','-','*','/','s','c','a','l','o','t','^','~']) do begin
          Operace(Znam[ZnamTop]); Dec(ZnamTop);
        end;
        Inc(ZnamTop); Znam[ZnamTop] := z; { operaci na zasobnik }
      end;
    end; { of case }
    bCont := Precti(r,z);
  end; { of while }

  { proved zbyle operace }
  while (ZnamTop > 0) and (Znam[ZnamTop] <> '(') do begin
    Operace(Znam[ZnamTop]); Dec(ZnamTop);
  end;
  Hodnota := Zas[1];
end;

procedure ZiskejPrubeh(var fce: funkce; defMin: real; defMax: real);
{ vytvori lin. spoj. seznam obsahujici jednotlive body zadane funkce
  na intervalu (defMin, defMax) }
var
  p,n: pzobrazeni;
  krok, x: real;
  def: Boolean;

begin
  { velikost kroku }
  krok := (defMax - defMin) / KROKU;
  { zaciname u leve hranice }
  x := defMin;
  new(n);
  fce.pZobr := n;
  n^.dalsi := nil;
  while x <= defMax do begin
    n^.x := x;
    n^.y := Hodnota(fce, n^.x, def);
    { je-li v danem bode definovana }
    if def then begin
      p := n;
      if (x + krok) <= defMax then begin
        new(n);
        p^.dalsi := n;
        n^.dalsi := nil;
        n^.x := x;
      end;
    end;
    x := x + krok;
  end;
end;

end.
