unit Kresli;

interface

uses
  Graph;

const
  PRIBLIZENIDEFAULT = 16;  { jeden pixel = interval velikosti 1/priblizeni }
  MAXZOOM = 200;
  HRANICE = 50;     { hodnoty nad tuto mez se nebudou vykreslovat }
  BARVAOS = 7;
  BARVAGRAFU = 15;
  ODSAZENI = 6;     { odsazeni popisku }
  RYSKA = 2;        { delka rysky na ose }
  POPISKUMIN = 8;   { minimalni pocet popisku na kazde ose }
  POPISKUMAX = 16;  { maximalni pocet popisku na kazde ose }
  ORIEBODUMIN = 32; { minimalni pocet orientacnich bodu na kazde ose }
  ORIEBODUMAX = 64; { maximalni pocet orientacnich bodu na kazde ose }
  DILKU = 8;        { pocet dilku na kazde ose }
  AVLEVO = -1; AVPRAVO = 1; ASTRED = 0; ANAHORU = -1; ADOLU = 1;
  { konstanty pro zarovnani popisku }

var
  iMaxX, iMaxY: LongInt;
  priblizeni: Integer;

procedure PridejPopisek(x: integer; y: integer; text: string; hzarovnani: integer; vzarovnani: integer; margin: integer);
  { umisti a zarovna popisek grafu }

procedure InicPlochu;
  { na kresli na plochu osy, a popisky }

procedure Caru(x1, y1, x2, y2: real);
  { vykresli caru, souradnice prevede na souradnice na obrazovce }

implementation

procedure PridejPopisek(x: integer; y: integer; text: string; hzarovnani: integer; vzarovnani: integer; margin: integer);
{ vlozi popisek na plochu, zarovna ho vzhledem k danym souradnicim a posune o dane odsazeni (margin) }
var w, h: integer;
begin
  w := TextWidth(text) div 2; h := TextHeight(text) div 2;
  x := x - w; y := y - h;
  x := x + hzarovnani * (w + margin);
  y := y + vzarovnani * (h + margin);
  OutTextXY(x,y,text);
end;

procedure InicPlochu;
{ nakresli na plochu osy a popisky }
var
  gMaxX, gMaxY: real;
  dDilku: Integer;
  i, j, l, skokX, skokY: Integer;
  s: string;
begin
  iMaxX := GetMaxX+1; iMaxY := GetMaxY+1;
  gMaxX := iMaxX / (PRIBLIZENI * 2);
  gMaxY := iMaxY / (PRIBLIZENI * 2);
  { dDilku := (ORIEBODUMIN+ORIEBODUMAX) div (trunc(gMaxX)*2); }
  SetColor(BARVAOS);
  skokX := (iMaxX div 2) div DILKU;
  skokY := (iMaxY div 2) div DILKU;
  for i := 1 to DILKU do begin
    j := i * skokX;
    l := i * skokY;
    Line(iMaxX div 2 - j, iMaxY div 2 - RYSKA, iMaxX div 2 - j, iMaxY div 2 + RYSKA);
    Line(iMaxX div 2 + j, iMaxY div 2 - RYSKA, iMaxX div 2 + j, iMaxY div 2 + RYSKA);
    str(j / PRIBLIZENI:0:1,s);
    PridejPopisek(iMaxX div 2 + j, iMaxY div 2, s, ASTRED, ADOLU, ODSAZENI);
    s := '-' + s;
    PridejPopisek(iMaxX div 2 - j, iMaxY div 2, s, ASTRED, ADOLU, ODSAZENI);

    Line(iMaxX div 2 - RYSKA, iMaxY div 2 - l, iMaxX div 2 + RYSKA, iMaxY div 2 - l);
    Line(iMaxX div 2 - RYSKA, iMaxY div 2 + l, iMaxX div 2 + RYSKA, iMaxY div 2 + l);
    str(l / PRIBLIZENI:0:1,s);
    PridejPopisek(iMaxX div 2, iMaxY div 2 - l, s, AVPRAVO, ASTRED, ODSAZENI);
    s := '-' + s;
    PridejPopisek(iMaxX div 2, iMaxY div 2 + l, s, AVPRAVO, ASTRED, ODSAZENI);
  end;
  PridejPopisek(iMaxX div 2, iMaxY div 2, '0', AVPRAVO, ADOLU, ODSAZENI);
  PridejPopisek(iMaxX div 2, 0, 'y', AVLEVO, ADOLU, ODSAZENI);
  PridejPopisek(iMaxX, iMaxY div 2, 'x', AVLEVO, ANAHORU, ODSAZENI);
  Line(iMaxX div 2, 0, iMaxX div 2, iMaxY);
  Line(0, iMaxY div 2, iMaxX, iMaxY div 2);
  SetColor(BARVAGRAFU);
end; { of InicPlochu }

function JeVHranicich(l: real): Boolean;
{ urci, zda je hodnota v rozmezi (-HRANICE, HRANICE)
  hodnoty mimo danou mez se nebudou kreslit pro zabraneni behove chybe }
begin
  JeVHranicich := (l > -HRANICE) and (l < HRANICE);
end;

function OOR(l: Longint): Boolean;
{ zjisti zda dana hodnota lezi v zobrazitelne oblasti }
begin
  OOR := not ((l < 0) or ((l > iMaxX) and (l > iMaxY)));
end;

procedure Caru(x1, y1, x2, y2: real);
{ prevede zadane souradnice na souradnice na obrazovce
  provede kontroly velikosti puvodnich i novych souradnic
  pro zabraneni behovym chybam vypoctu a vykreslovani }
var x1i, x2i, y1i, y2i: Longint;
  s: string;
begin
  if (JeVHranicich(y1) and JeVHranicich(y2)) then begin
    x1i := round(x1*PRIBLIZENI)+iMaxX div 2;
    y1i := -round(y1*PRIBLIZENI)+iMaxY div 2;
    x2i := round(x2*PRIBLIZENI)+iMaxX div 2;
    y2i := -round(y2*PRIBLIZENI)+iMaxY div 2;

    if (OOR(x1i) and OOR(y1i)) and (OOR(x2i) and OOR(y2i)) then
      line(x1i, y1i, x2i, y2i);
  end;
end;

begin
  priblizeni := PRIBLIZENIDEFAULT;
end.
